mod namespace;
mod manage_interfaces;
mod wireguard_manager;
mod config;

use rtnetlink::NetworkNamespace;
use futures::executor::block_on;

fn main() {
    env_logger::Builder::from_default_env()
        .format_timestamp_secs()
        .filter(None, log::LevelFilter::Debug)
        .init();
    let args: Vec<String> = std::env::args().collect();
    match args.len() {
        3 => {
            let cmd = &args[1];
            let param = &args[2];
            match &cmd[..] {
                "-conf" => {
                    for vpn in config::get_vpn_conf(param.to_string()) {
                        create_namespace(vpn.get_consumable());
                    }
                }
                _ => {},
            }
        },
        _ => {},
    };
}

fn create_namespace(vpn: config::ConsumableVPNConfig) {
    let ns_name = vpn.namespace_name.clone();
    block_on(NetworkNamespace::add(ns_name.clone())).unwrap();
    namespace::create_ns::run_in_namespace(|| {
            manage_interfaces::set_interface_lo_up().unwrap();
            },
        &ns_name).unwrap();

    manage_interfaces::create_and_move_wireguard_interface(
        vpn.interface_name.clone(),
        ns_name.clone()
    ).unwrap();
    namespace::create_ns::run_in_namespace(|| {
            manage_interfaces::configure_wireguard_interface(vpn.interface_name.clone(),
                vpn.ip.clone(),
                vpn.endpoint.ip.clone(),
                vpn.prefix as u8,
                vpn.public_key,
                vpn.private_key).unwrap();
            },
        &ns_name).unwrap();
}

