use rtnetlink::{new_connection, Error};
use std::os::fd::AsRawFd;
use futures::TryStreamExt;
mod netlink;

pub fn create_and_move_wireguard_interface(
    interface_name: String,
    namespace_name: String,
) -> Result<(), Error> {
    tokio::runtime::Runtime::new().unwrap().handle().block_on( async {
        let (connection, handle, _) = new_connection().unwrap();
        tokio::spawn(connection);
        netlink::create_wireguard_interface(handle.clone(), interface_name.clone()).await?;
        let link = netlink::get_link_interface(handle.clone(), interface_name.clone()).await?;
        handle.link().set(link.header.index).setns_by_fd(crate::namespace::create_ns::get_ns_fd(&namespace_name).unwrap().as_raw_fd()).execute().await?;
        Ok(())
    })
}

pub fn configure_wireguard_interface(
    interface_name: String,
    interface_ip: String,
    peer_ip: String,
    prefix: u8,
    wg_pub_key: [u8; 32],
    wg_priv_key: [u8;32]) -> Result<(), Error> {
    tokio::runtime::Runtime::new().unwrap().handle().block_on( async {
        let (connection, handle, _) = new_connection().unwrap();
        tokio::spawn(connection);
        let link = netlink::get_link_interface(handle.clone(), interface_name.clone()).await?;
        netlink::assign_ip(handle.clone(), link.clone(), interface_ip, prefix).await?;
        crate::wireguard_manager::add_properties::set_params(wg_pub_key, wg_priv_key, peer_ip, interface_name.clone());
        netlink::set_interface_up(handle, link).await?;
        Ok(())
    })
}

pub fn set_interface_lo_up() -> Result<(), Error> {
    tokio::runtime::Runtime::new().unwrap().handle().block_on( async {
        let (connection, handle, _) = new_connection().unwrap();
        tokio::spawn(connection);
        let link = netlink::get_link_interface(handle.clone(), String::from("lo")).await?;
        netlink::set_interface_up(handle, link).await?;
        Ok(())
    })
}

pub fn get_inferfaces() -> Result<(), Error> {
        tokio::runtime::Runtime::new().unwrap().handle().block_on( async {
        let (connection, handle, _) = new_connection().unwrap();
        tokio::spawn(connection);
        log::debug!("ARE WE STOPPING YET???");
        let mut links = handle.link().get().match_name("lo".to_string()).execute();
        if let Some(link) = links.try_next().await.unwrap() {
            let index = link.header.index;
                log::debug!("index:{}", index);
        } else {
            println!("no link link lo found");
        }
        });
    Ok(())
}

