use rtnetlink::{Error, Handle};
use netlink_packet_route::link::LinkMessage;
use std::net::IpAddr;
use futures::TryStreamExt;

pub async fn set_interface_up(handle: Handle,
    link: LinkMessage) -> Result<(), Error> {
    let index = link.header.index;
    log::debug!("index: {}", index);
    handle
        .link()
        .set(index)
        .up()
        .execute()
        .await
}

pub async fn create_wireguard_interface(
    handle: Handle,
    interface: String) -> Result<(), Error> {
    handle.link().add().wireguard(interface).execute().await
}

pub async fn assign_ip(
    handle: Handle,
    link: LinkMessage,
    ip: String,
    prefix: u8) -> Result<(), Error> {
    let usable_ip: IpAddr = match ip.parse() {
        Ok(ip_ok) => ip_ok,
        Err(_e) => {
            log::debug!("malformed ip");
            return Err(Error::InvalidIp(ip.into_bytes()));
        },
    };
    handle
        .address()
        .add(link.header.index, usable_ip, prefix)
        .execute()
        .await
}

pub async fn get_link_interface(
    handle: Handle,
    interface: String) -> Result<LinkMessage ,Error> {
    let mut links = handle.link().get().match_name(interface).execute();
    if let Some(link) = links.try_next().await.unwrap() {
        Ok(link)
    } else {
        log::debug!("no link link lo found");
        Err(Error::RequestFailed)
    }
}
