#include "wireguard_wrapper.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>

void generate_default_ipv4(wg_allowedip *ip);
void generate_default_ipv6(wg_allowedip *ip);
wg_endpoint generate_endponit(char *ip, unsigned short port);

wg_peer generate_new_peer(wg_key *pub_key, char *ip, unsigned short port) {
  wg_allowedip *ipv4, *ipv6;
  ipv4 = malloc (sizeof (wg_allowedip));
  ipv6 = malloc (sizeof (wg_allowedip));
  generate_default_ipv4(ipv4);
  generate_default_ipv6(ipv6);
  ipv4->next_allowedip = ipv6;

  wg_peer new_peer = {
		.flags = WGPEER_HAS_PUBLIC_KEY,
    .first_allowedip = ipv4,
    .last_allowedip = ipv6,
    .endpoint = generate_endponit (ip, port),
	};
  printf("protocol 0.101: %d\n",new_peer.endpoint.addr.sa_family);
  printf("protocol 0.102: %d\n",new_peer.endpoint.addr4.sin_family);
  memcpy(new_peer.public_key, pub_key, sizeof (wg_key));
  printf("protocol 0.111: %d\n",new_peer.endpoint.addr.sa_family);
  printf("protocol 0.112: %d\n",new_peer.endpoint.addr4.sin_family);
  return new_peer;
}

wg_device generate_new_device(wg_key *priv_key,
                              char device_name[IFNAMSIZ],
                              unsigned short port,
                              wg_peer peer)
{
  wg_device new_device = {
    .listen_port = port,
    .flags = WGDEVICE_HAS_PRIVATE_KEY | WGDEVICE_HAS_LISTEN_PORT,
    .first_peer = &peer,
    .last_peer = &peer,
  };
  memcpy(new_device.name, device_name, sizeof (char[IFNAMSIZ]));
  memcpy(new_device.private_key, priv_key, sizeof (wg_key));
  printf("str2: %s\n",new_device.name);
  wg_key_b64_string key, key2;
  wg_key_to_base64(key, new_device.first_peer->public_key);
  printf("str2: %s\n",key);
  wg_key_to_base64(key2, new_device.private_key);
  printf("str3: %s\n",key2);
  printf("port: %d\n",new_device.first_peer->endpoint.addr4.sin_port);
  printf("protocol 1: %d\n",new_device.first_peer->endpoint.addr.sa_family);
  printf("protocol 2: %d\n",new_device.first_peer->endpoint.addr4.sin_family);
  char str[INET_ADDRSTRLEN];
  inet_ntop(AF_INET, &(new_device.first_peer->endpoint.addr4.sin_addr), str, INET_ADDRSTRLEN);
  printf("ip: %s\n", str);
  return new_device;
}

wg_endpoint generate_endponit(char *ip, unsigned short port) {
  struct in_addr ipv4;
  inet_pton (AF_INET, ip, &ipv4);
  struct sockaddr_in address = {
    .sin_family = AF_INET,
    .sin_port = htons(port),
    .sin_addr = ipv4,
  };

  wg_endpoint ret = {
    .addr4 = address,
  };

  printf("protocol 0.1: %d\n",ret.addr.sa_family);
  printf("protocol 0.2: %d\n",ret.addr4.sin_family);

  return ret;
}

void generate_default_ipv4(wg_allowedip *ip) {
  struct in_addr ipv4;
  inet_pton (AF_INET, "0.0.0.0", &ipv4);

  *ip = (struct wg_allowedip) {
    .family = AF_INET,
    .ip4 = ipv4,
  };
}

void generate_default_ipv6(wg_allowedip *ip) {
  struct in6_addr ipv6;
  inet_pton (AF_INET6, "::0", &ipv6);
  *ip = (struct wg_allowedip) {
    .family = AF_INET6,
    .ip6 = ipv6,
  };
}

void clean_device(wg_peer *peer) {
  free(peer->first_allowedip);
  free(peer->last_allowedip);
}

void list_devices(void)
{
	char *device_names, *device_name;
	size_t len;

	device_names = wg_list_device_names();
	if (!device_names) {
		perror("Unable to get device names");
		exit(1);
	}
	wg_for_each_device_name(device_names, device_name, len) {
		wg_device *device;
		wg_peer *peer;
		wg_key_b64_string key;

		if (wg_get_device(&device, device_name) < 0) {
			perror("Unable to get device");
			continue;
		}
		if (device->flags & WGDEVICE_HAS_PUBLIC_KEY) {
			wg_key_to_base64(key, device->public_key);
			printf("%s has public key %s\n", device_name, key);
		} else
			printf("%s has no public key\n", device_name);
		wg_for_each_peer(device, peer) {
			wg_key_to_base64(key, peer->public_key);
			printf(" - peer %s\n", key);
		}
		wg_free_device(device);
	}
	free(device_names);
}

int it_works() {
  list_devices();
  return 5;
}
